/*
 *  OGLTestView.cpp
 *  OpenGLHIView
 *
 *  Created by Arne Scheffler on 8/19/06.
 *  Copyright 2006 Arne Scheffler. All rights reserved.
 *
 */

#include "OGLTestView.h"

#include <OpenGL/glu.h>

//-------------------------------------------------------------------------------------------
static void MyTimer (EventLoopTimerRef inTimer, void *inUserData)
{
	OGLTestView* view = (OGLTestView*)inUserData;
	view->onTimer ();
}

static int gRandInit = 0;

//-------------------------------------------------------------------------------------------
OGLTestView::OGLTestView (HIViewRef view)
: OpenGLHIView (view)
{
	srandom (gRandInit++);
	
	sinus = sinus2 = 0.f;
	add1 = ((float)random () / (float)INT_MAX) * 0.04;
	add2 = ((float)random () / (float)INT_MAX) * 0.04;
	mul1 = mul2 = 1.f;
	InstallEventLoopTimer (GetMainEventLoop (), kEventDurationMillisecond, kEventDurationSecond/60, MyTimer, this, &timer);
}

//-------------------------------------------------------------------------------------------
OGLTestView::~OGLTestView ()
{
	RemoveEventLoopTimer (timer);
}

//-------------------------------------------------------------------------------------------
void OGLTestView::SetupOpenGL ()
{
	// we enable antialiasing and syncing to the vertical retrace 
	glMatrixMode (GL_PROJECTION);
	glEnable (GL_LINE_SMOOTH);
	glEnable (GL_BLEND);
	glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glHint (GL_LINE_SMOOTH_HINT, GL_DONT_CARE); 
	glClearColor (0.0,0.0,0.0,0.0);
	GLint sync = 1;
	aglSetInteger (getAGLContext (), AGL_SWAP_INTERVAL, &sync);
}

//-------------------------------------------------------------------------------------------
void OGLTestView::OpenGLBoundsChanged ()
{
	// we setup the opengl matrix to be like a normal HIView in this test case
 	HIRect r = Bounds ();
	glLoadIdentity (); 
	gluOrtho2D (r.origin.x,r.size.width,r.size.height,r.origin.y);
}

//-------------------------------------------------------------------------------------------
void OGLTestView::DrawOpenGL ()
{
	// just draw ;-)

	HIRect r = Bounds ();

	glColor3f (0.f, 0.f, 1.f);
	glLineWidth (1);

	float x1,y1,x2,y2;

	float x = sin(sinus*M_PI);
	float xx = sin(sinus2*M_PI);
	
	glBegin (GL_LINES);
		x1 = x * r.size.width;
		y1 = r.origin.y;
		x2 = r.size.width - x * r.size.width;
		y2 = r.size.height;
		glVertex2f (x1, y1);
		glVertex2f (x2, y2);
		glVertex2f (x1+3, y1);
		glVertex2f (x2-3, y2);
		glVertex2f (x1-3, y1);
		glVertex2f (x2+3, y2);
		x1 = r.origin.x; 
		y1 = xx * r.size.height;
		x2 = r.size.width;
		y2 = r.size.height - xx * r.size.height;
		glVertex2f (x1, y1);
		glVertex2f (x2, y2);
		glVertex2f (x1, y1+3);
		glVertex2f (x2, y2-3);
		glVertex2f (x1, y1-3);
		glVertex2f (x2, y2+3);
	glEnd ();

}

//-------------------------------------------------------------------------------------------
void OGLTestView::onTimer ()
{
	// we use a timer to draw into the OpenGL context
	sinus += add1;
	if (sinus > 1.f)
	{
		sinus = 0;
	}
		add1 += 0.0002f * mul1;
		if (add1 > 0.04)
			mul1 = -1.f;
		if (add1 < 0.001)
			mul1 = 1.f;
	
	sinus2 += add2;
	if (sinus2 > 1.f)
	{
		sinus2 = 0;
	}
		add2 += 0.0002f * mul2;
		if (add2 > 0.04)
			mul2 = -1.f;
		if (add2 < 0.001)
			mul2 = 1.f;

	// we don't use TView::Invalidate(), it would cost too much overhead
	if (getAGLContext ())
	{
		aglSetCurrentContext (getAGLContext ());
		glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

		DrawOpenGL ();

		aglSwapBuffers (getAGLContext ());
	}
}

//-------------------------------------------------------------------------------------------
ControlPartCode	OGLTestView::HitTest (const HIPoint& inWhere)
{
	// we report that we want always get hit
	return kControlContentMetaPart;
}

//-------------------------------------------------------------------------------------------
OSStatus OGLTestView::ControlHit (ControlPartCode inPart, UInt32 inModifiers)
{
	// We change opacity when clicked
	setOpaque (!isOpaque ());
	if (!isOpaque ())
		Invalidate ();
	return noErr;
}

//-------------------------------------------------------------------------------------------
void OGLTestView::Draw (RgnHandle inLimitRgn, CGContextRef inContext)
{
	// You can also draw into the HIView here, but first we call the superclass
	OpenGLHIView::Draw (inLimitRgn, inContext);
	
	if (!isOpaque ()) // and we don't need to draw if the opengl context is opaque
	{
		CGContextSetRGBFillColor (inContext, 0.0, 0.5, 0.0, 0.5);
		CGContextFillRect (inContext, Bounds ());
	}
}

//-------------------------------------------------------------------------------------------
// The Following stuff is mandatory for TView subclasses
//-------------------------------------------------------------------------------------------
void OGLTestView::RegisterClass ()
{
	static bool sRegistered;
	if (!sRegistered)
	{
		CFStringRef classID = CFStringCreateWithCString (NULL, "de.knup.arne.ogltestview", kCFStringEncodingUTF8);
		TView::RegisterSubclass (classID, Construct);
		CFRelease (classID);
		sRegistered = true;
	}
}

//-------------------------------------------------------------------------------------------
OSStatus OGLTestView::Construct (HIObjectRef inObjectRef, TObject** outObject)
{
	*outObject = new OGLTestView ((HIViewRef) inObjectRef);
	return noErr;
}

//-------------------------------------------------------------------------------------------
ControlKind OGLTestView::GetKind ()
{
	const ControlKind kOpenGLHIViewKind = { 'knup', 'OGTV' };
	return kOpenGLHIViewKind;
}

//-------------------------------------------------------------------------------------------
// One Time HIView registerer
//-------------------------------------------------------------------------------------------
class OGLTestViewRegisterer
{
public:
	OGLTestViewRegisterer () { OGLTestView::RegisterClass (); }
};

OGLTestViewRegisterer _gOGLTestViewRegisterer;

