/*
 * OpenGLHIView.h
 *
 * Copyright (c) 2006, Arne Scheffler
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that
 * the following conditions are met:
 * 
 * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 * following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 * following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Author nor the names of its contributors may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#include <Carbon/Carbon.h>
#include <AGL/AGL.h>

#include </Developer/Examples/Carbon/HIFramework/TView.h>

//-------------------------------------------------------------------------------------------
class OpenGLHIView : public TView
{
	// OpenGLHIView methods
//-------------------------------------------------------------------------------------------
protected:
	// override to setup the pixelformat differently
	virtual const GLint* GetOpenGLAttributes ();

	// draw into the current context
	virtual void DrawOpenGL () {}

	// Setup OpenGL
	virtual void SetupOpenGL () {}
	virtual void OpenGLBoundsChanged () {}
	void setOpaque (bool state);

	// member access
	bool isOpaque () const { return opaque; }
	AGLContext getAGLContext () const { return aglContext; }
	AGLPixelFormat getAGLPixelFormat () const { return pixelFormat; }

//-------------------------------------------------------------------------------------------
private:
	OSStatus CreateOpenGLContextForWindow (WindowRef window);
	void CleanupOpenGL ();
	void SetupOpenGLBounds ();

	static OSStatus WindowEventHandler (EventHandlerCallRef inHandler, EventRef inEvent, void* inUserData);

	AGLContext aglContext;
	AGLPixelFormat pixelFormat;
	EventHandlerRef windowEventHandler;
	EventHandlerUPP windowEventHandlerUPP;
	bool isResizing;
	Boolean opaque;

//-------------------------------------------------------------------------------------------
protected:
	OpenGLHIView (HIViewRef inView);
	~OpenGLHIView ();

	// TView overrides
	void OwningWindowChanged (WindowRef oldWindow, WindowRef newWindow);
	void BoundsChanged (UInt32 inOptions, const HIRect& inOriginalBounds, const HIRect& inCurrentBounds );
	void Draw (RgnHandle inLimitRgn, CGContextRef inContext);

	OSStatus Initialize (TCarbonEvent& inEvent);
};
